#include <format>

#include "gui/FontManager.hpp"
#include "Artboards.hpp"
#include "Event.hpp"
#include "gui/theme.hpp"

namespace mcpaint {

using namespace gui;
using namespace events;
  
Artboards::Artboards() {
	this->artboards.push_back(Artboard({
		.position = {0, 0},
		.size     = {1920.0f, 1080.0f},
		.canvas   = LoadRenderTexture(1920, 1080),
		.scale    = 1.0f
	}));
}

void Artboards::initialScale() {
	if (this->size_.x == 0.0f || this->artboards.size() < 1)
		return;

	float max = this->artboards.at(0).size.x;

	this->scale = this->size_.x * .9f / max;
}

void Artboards::update(float deltaTime, const gui::view* const parent) {
	if (this->scale == 0.0f)
		this->initialScale();

	BeginTextureMode(this->artboards.at(this->currentArtboard).canvas);
		ClearBackground(WHITE);
		DrawPixel(100, 100, BLACK);
		DrawCircle(100, 100, 50, RED);
	EndTextureMode();

	Vector2 scroll = GetMouseWheelMoveV();

	if (scroll.y != 0) {
		if (this->scale + scroll.y / 100 > 0.05f && this->scale + scroll.y / 100 <= 3.0f)
			this->scale += scroll.y / 100;
	}

	Vector2 mouse = GetMousePosition();
	if (this->hovered()) {
		if (IsKeyDown(KEY_SPACE)) {
			dispatch << Event(Event::Type::POINTER_REQUEST, mouse);

			if (IsMouseButtonDown(MOUSE_LEFT_BUTTON)) {
				Vector2 mouseDelta = GetMouseDelta();

				if (mouseDelta.x != 0 || mouseDelta.y != 0)
					this->offset += mouseDelta;
			}
		} else if (IsMouseButtonDown(MOUSE_LEFT_BUTTON)) {
			size_t i = 0;
			for (const auto& artboard : this->artboards) {
				if (CheckCollisionPointRec(mouse, {artboard.position.x, artboard.position.y, artboard.size.x * this->scale, artboard.size.y * this->scale})) {
					this->currentArtboard = i;
					break;
				}
				i++;
			}
		}
	}
}

Artboards::Artboards(Artboard initialArtboard) {
	this->artboards.push_back(std::move(initialArtboard));
}

void Artboards::render(float deltaTime) {
	FontManager& fontManager = FontManager::Instance();

	BeginScissorMode(this->position_.x, this->position_.y, this->size_.x, this->size_.y);

	std::string hud = std::format(
		"x: {}, y: {}\nwidth: {}, height: {}\nscale: {}\noffset: {}, {}",
		this->position_.x,
		this->position_.y,
		this->size_.x,
		this->size_.y,
		this->scale,
		this->offset.x, this->offset.y
	);

	size_t i = 0;
	float localOffset = 0;
	for (auto& artboard : this->artboards) {
		Vector2 scaled   = artboard.size * this->scale;
		Vector2 position = this->position_ + this->size_ * .5f - scaled * .5f + this->offset;
		//DrawRectangleV(position, scaled, WHITE);
		DrawTexturePro(artboard.canvas.texture, {0, 0, static_cast<float>(artboard.canvas.texture.width), -static_cast<float>(artboard.canvas.texture.height)}, {position.x, position.y, scaled.x, scaled.y}, {0, 0}, 0, WHITE);

		float border = 2;
		if (this->currentArtboard == i)
			DrawRectangleLinesEx({position.x - border * .5f - 3, position.y - border * .5f - 3, scaled.x + border + 6, scaled.y + border + 6}, border, SKYBLUE);
		i++;
	}



	Vector2 bounds = MeasureTextEx(
		fontManager.get("assets/fonts/sf-pro.otf", 20),
		hud.c_str(),
		20,
		0
	);
	DrawRectangle(
		this->position_.x,
		this->position_.y,
		bounds.x + 20,
		bounds.y + 20,
		ColorFromHex(0x00000066)
	);
	DrawTextPro(
		fontManager.get("assets/fonts/sf-pro.otf", 20),
		hud.c_str(),
		{this->position_.x + 10, this->position_.y + 10},
		{0, 0},
		0,
		20,
		0,
		WHITE
	);

	EndScissorMode();
}

}