#pragma once

#include <raylib.h>
#include <memory>
#include <unordered_map>
#include <vector>
#include <functional>

namespace mcpaint::events {

struct Event {
	enum class Type {
		CLICK,
		POINTER_REQUEST
	};

	Type type;
	Vector2 mouse;
};

using EventCallback = std::function<void(Event)>;

class Dispatch {
	private:
		std::unordered_map<Event::Type, std::vector<EventCallback>> handlers;
		static std::unique_ptr<Dispatch> Instance_;
		Dispatch() = default;
	public:
		static Dispatch& Instance() {
			if (Dispatch::Instance_ == nullptr)
				Dispatch::Instance_ = std::unique_ptr<Dispatch>(new Dispatch());

			return *Dispatch::Instance_.get();
		}

		void addListener(Event::Type type, EventCallback callback) {
			this->handlers[type].push_back(callback);
		}

		const Dispatch& operator <<(const Event& event) const {
			for (const auto& handler : this->handlers.at(event.type)) {
				handler(event);
			}

			return *this;
		}

		Dispatch(const Dispatch&) = delete;
		Dispatch& operator=(const Dispatch&) = delete;
		Dispatch(Dispatch&&) = delete;
		Dispatch& operator=(Dispatch&&) = delete;
};

inline const Dispatch& dispatch = Dispatch::Instance();

}