#include "UndoRedoManager.hpp"

#include <cstdio>

namespace mcpaint {

UndoRedoManager::UndoRedoManager() : pointer(-1), size(0) {
}

void UndoRedoManager::push(const RenderTexture2D& texture) {
	Image image = LoadImageFromTexture(texture.texture); // getting the current buffer
	// if the stack is full, remove the first element and shift subsequent images
	if (this->pointer >= 0 && this->pointer >= UndoRedoManager::MAX_UNDO_REDO - 1) {
	printf("PUSH %u, max: %u\n", this->pointer, UndoRedoManager::MAX_UNDO_REDO);
		if (this->stack[0].has_value())
			UnloadImage(this->stack[0].value());
		for (size_t i = 0; i < this->size; ++i)
			this->stack[i] = this->stack[i + 1];
		
		this->pointer = UndoRedoManager::MAX_UNDO_REDO - 1;
		this->size    = UndoRedoManager::MAX_UNDO_REDO;

		this->stack[this->pointer] = image;
	} else if (this->pointer >= 0 && this->pointer + 1 < this->size) {
		this->pointer++;
		// here we clear all subsequent images, because the stack is broken if a
		// push is performed in the middle of the stack
		for (size_t i = this->pointer; i < this->size; ++i) {
			if (!this->stack[i].has_value())
				continue;
			UnloadImage(this->stack[i].value());
			this->stack[i] = std::nullopt;
		}
		this->size = this->pointer + 1;
		this->stack[this->pointer] = image;
	} else {
		this->stack[++this->pointer] = image;
		this->size++;
	}
}

void UndoRedoManager::undo(const RenderTexture2D& canvas) {
	if (this->pointer > 0)
		this->pointer--;

	if (this->stack[this->pointer].has_value())
		UndoRedoManager::RenderImage(canvas, this->stack[this->pointer].value());
}

void UndoRedoManager::redo(const RenderTexture2D& canvas) {
	if (this->pointer + 1 < this->size)
		this->pointer++;

	if (this->stack[this->pointer].has_value())
		UndoRedoManager::RenderImage(canvas, this->stack[this->pointer].value());
}

void UndoRedoManager::RenderImage(const RenderTexture2D& canvas, const Image& image) {
	Texture2D img = LoadTextureFromImage(image);
	BeginTextureMode(canvas);
		DrawTexturePro(img, {0, 0, (float)img.width, (float)-img.height }, {0, 0, static_cast<float>(img.width), static_cast<float>(img.height)}, {0, 0}, 0, WHITE);
	EndTextureMode();
	UnloadTexture(img);
}

}