#pragma once;

#include <type_traits>
#include <optional>
#include <raylib.h>

namespace mcpaint {

class UndoRedoManager {
	private:
	 	constexpr static size_t MAX_UNDO_REDO = 50;
		std::optional<Image> stack[UndoRedoManager::MAX_UNDO_REDO];
		int64_t pointer;
		size_t size;
	public:
		UndoRedoManager();
		~UndoRedoManager() = default;

		void push(const RenderTexture2D& texture);
		void undo(const RenderTexture2D& canvas);
		void redo(const RenderTexture2D& canvas);

		bool canUndo() const {
			return this->pointer > 0;
		}

		bool canRedo() const {
			return this->pointer + 1 < this->size;
		}

		static void RenderImage(const RenderTexture2D& canvas, const Image& image);

		UndoRedoManager(const UndoRedoManager&) = delete;
		UndoRedoManager& operator=(const UndoRedoManager&) = delete;

		UndoRedoManager(UndoRedoManager&&) = delete;
		UndoRedoManager& operator=(UndoRedoManager&&) = delete;
};

}