#include "Button.hpp"
#include "FontManager.hpp"

namespace mcpaint::gui {

button::button(std::string label, ButtonCallback callback) : 
		label_(std::move(label)),
		callback(std::move(callback)) {
	this->spec_.display         = Display::INLINE;
	this->spec_.padding         = 5;
	this->spec_.color           = Theme::ELEMENT_FG;
	this->spec_.background      = Theme::ELEMENT;
	this->spec_.hoverBackground = Theme::ELEMENT_HOVER;
	this->spec_.radius          = 3;
}
button::button(Icon icon, ButtonCallback callback) : 
		isIcon(true),
		icon(icon),
		callback(std::move(callback)) {
	this->spec_.display         = Display::INLINE;
	this->spec_.padding         = 5;
	this->spec_.color           = Theme::ELEMENT_FG;
	this->spec_.background      = Theme::ELEMENT;
	this->spec_.hoverBackground = Theme::ELEMENT_HOVER;
	this->spec_.radius          = 3;
}
 
void button::update(float deltaTime, const view* const parent) {
	Vector2 mouse = GetMousePosition();

	if (this->hovered()) {
		if (IsMouseButtonPressed(MOUSE_LEFT_BUTTON)) {
			Event event = {Event::Type::CLICK, mouse};
			this->callback(event);
		}
	}

	view::update(deltaTime, parent);
}

void button::render(float deltaTime) {
	FontManager& fontManager = FontManager::Instance();
	const Font& font         = fontManager.get(this->spec_.font, this->spec_.fontSize);

	view::render(deltaTime);
	float   textWidth    = this->contentWidth();
	Vector2 textPosition = {
		round(this->position_.x + (this->size_.x + this->spec_.padding * 2) / 2 - textWidth * .5f),
		round(this->position_.y + this->size_.y / 2 - this->spec_.fontSize * .5f)
	};

	if (this->active() || this->hovered()) {
		Color color = this->active() ? this->spec_.activeBackground : this->spec_.hoverBackground;

		DrawRectangleRounded({this->position_.x, this->position_.y, this->size_.x, this->size_.y}, this->roundness(), 0, color);
		DrawRectangleRoundedLinesEx({this->position_.x, this->position_.y, this->size_.x, this->size_.y}, this->roundness(), 0, 1, ColorFromHex(0x636363FF));
	}

	if (this->isIcon)
		DrawIcon(this->icon, textPosition, this->spec_.fontSize, this->spec_.color);
	else
		DrawTextPro(font, this->label_.c_str(), textPosition, {0, 0}, 0, this->spec_.fontSize, 0, this->spec_.hoverColor);
}

size_t button::contentWidth() const {
	if (this->isIcon)
		return MeasureIcon(this->icon, this->spec_.fontSize).x + this->spec_.padding * 2;

	FontManager& fontManager = FontManager::Instance();
	const Font& font         = fontManager.get(this->spec_.font, this->spec_.fontSize);

	return MeasureTextEx(font, this->label_.c_str(), this->spec_.fontSize, 0).x + this->spec_.padding * 2;
}

}