#pragma once

#include <raylib.h>
#include <variant>
#include <optional>

#include "../util.hpp"

namespace mcpaint::gui {

using Value = std::variant<std::monostate, float, int>;

struct Vec2 {
	Value x;
	Value y;

	Vector2 toPixels() const {
		Vector2 windowSize = GetWindowSize();
		Vector2 outValue;

		if (std::holds_alternative<float>(this->x))
			outValue.x = std::get<float>(this->x) * windowSize.x / 100.0f;
		else
			outValue.x = std::get<int>(this->x);

		if (std::holds_alternative<float>(this->y))
			outValue.y = std::get<float>(this->y) * windowSize.y / 100.0f;
		else
			outValue.y = std::get<int>(this->y);

		return outValue;
	}
};

class Element {
	protected:
		Vec2 position;
		Vec2 size;
		std::optional<const Element*> parent;
	public:
		Element(Vec2 position, Vec2 size);
		Element(Vec2 position, Vec2 size, const Element* parent);
		virtual ~Element() = default;

		virtual void update(float deltaTime);
		virtual void render(float deltaTime);

		Vector2 getPosition() const {
			return this->position.toPixels();
		}

		Vector2 getSize() {
			return this->size.toPixels();
		}

		Element(const Element&) = delete;
		Element& operator=(const Element&) = delete;

		Element(Element&&) = delete;
		Element& operator=(Element&&) = delete;
};

}