#include "FontManager.hpp"

#include "Icon.hpp"

namespace mcpaint::gui {

std::unique_ptr<FontManager> FontManager::Instance_ = nullptr;
Font FontManager::IconFont_ = {0};

FontManager::FontManager() {
	FontManager::IconFont_ = LoadFontEx(
		"assets/fonts/icons-solid.otf",
		FontManager::ICON_SIZE * 2, // HiDPI
		const_cast<int*>(Iconset),
		sizeof(Iconset) / sizeof(Iconset[0])
	);
}

FontManager::~FontManager() {
	for (auto& font : this->fonts) {
		for (auto& size : font.second)
			UnloadFont(size.second);
	}
}

Font FontManager::get(std::string path, size_t size) {
	size *= 2; // this is for high DPI
	if (this->fonts.find(path) == this->fonts.end())
		this->fonts.emplace(path, std::unordered_map<size_t, Font>({{size, LoadFontEx(path.c_str(), size, nullptr, 0)}}));
	else if (this->fonts.at(path).find(size) == this->fonts.at(path).end())
		this->fonts.at(path).emplace(size, LoadFontEx(path.c_str(), size, nullptr, 0));
	
	SetTextureFilter(this->fonts.at(path).at(size).texture, TEXTURE_FILTER_BILINEAR);

	return this->fonts.at(path).at(size);
}

}