#pragma once

#include <raylib.h>
#include <string>
#include <unordered_map>

namespace mcpaint::gui {

class FontManager {
	protected:
		constexpr static size_t ICON_SIZE = 50;
		std::unordered_map<std::string, std::unordered_map<size_t, Font>> fonts;

		FontManager();
		static std::unique_ptr<FontManager> Instance_;
		static Font IconFont_;
	public:
		~FontManager();

		Font get(std::string path, size_t size);
		
		static FontManager& Instance() {
			if (FontManager::Instance_ == nullptr)
				FontManager::Instance_ = std::unique_ptr<FontManager>(new FontManager());

			return *FontManager::Instance_.get();
		}

		inline static const Font& IconFont() {
			return FontManager::IconFont_;
		}

		FontManager(const FontManager&) = delete;
		FontManager& operator=(const FontManager&) = delete;
		FontManager(FontManager&&) = delete;
		FontManager& operator=(FontManager&&) = delete;
};

}